/*
 * Copyright (C) 2008 Emweb bvba, Kessel-Lo, Belgium.
 *
 * See the LICENSE file for terms of use.
 */
#include "Wt/WApplication"
#include "Wt/WViewWidget"

#include "WebSession.h"
#include "DomElement.h"

namespace Wt {

WViewWidget::WViewWidget(WContainerWidget *parent)
  : WWebWidget(parent),
    contents_(0)
{ }

void WViewWidget::load()
{
  if (!contents_) {
    update();
    needContentsUpdate_ = false; // not update but create
  }

  WWebWidget::load();
}

void WViewWidget::update()
{
  assert(!contents_);

  WApplication::instance()->setExposeSignals(false);
  contents_ = renderView();
  needContentsUpdate_ = true;
  WApplication::instance()->setExposeSignals(true);

  addChild(contents_);

  setInline(contents_->isInline());

  repaint(RepaintInnerHtml);
}

void WViewWidget::prepareRerender()
{
  if (!contents_) {
    update();
    needContentsUpdate_ = false; // not update but create
  }

  WWebWidget::prepareRerender();
}

void WViewWidget::doneRerender()
{
  setIgnoreChildRemoves(true);
  delete contents_;
  contents_ = 0;
  setIgnoreChildRemoves(false);
}

void WViewWidget::updateDom(DomElement& element, bool all)
{
  if (contents_ && (needContentsUpdate_ || all)) {
    bool savedVisibleOnly =
      WApplication::instance()->session()->renderer().visibleOnly();
    
    WApplication::instance()->session()->renderer()
      .setVisibleOnly(false);

    DomElement *e = contents_->webWidget()->createDomElement();

    if (!all)
      element.setWasEmpty(true); // removes previous content
    element.addChild(e);

    WApplication::instance()->session()->renderer()
      .setVisibleOnly(savedVisibleOnly);

    needContentsUpdate_ = false;
  }

  WWebWidget::updateDom(element, all);
}

DomElementType WViewWidget::domElementType() const
{
  return isInline() ? DomElement_SPAN : DomElement_DIV;
}

}
